/*	Fichero dialogo_control_.h
	
	Contiene el interfaz para el cuadro de dialogo que permite configurar
	un _control_	
	
	Ultima actualizacion: Viernes 6-10-2006
*/


#ifndef _dialogo_control__h
#define _dialogo_control__h

#include "wx/wx.h"
#include "wx/statline.h"
#include "wx/spinctrl.h"
#include "wx/valgen.h"
/*	Este fichero es necesario para que el cuadro de dialogo pueda manejar
	los mismos tipos de seales que manejan los elementos */
#include "senal.h"
#include "constantes.h"
#include "dialogo.h"


/* Identificadores numricos asignados a los botones del cuadro de dialogo*/
enum {
	ID_DIALOGO__control_=7566,
	ID_NOMBRE,
	ID_VOLTAJE_ACTIVACION,	ID_VOLTAJE_REPOSO, 
	ID_CONEXION_REPOSO, ID_CONEXION_ACTIVACION
};

class Dialogo_control_ : public Dialogo
{
	DECLARE_CLASS (Dialogo_control_)
	DECLARE_EVENT_TABLE()
	public: 
		Dialogo_control_ ();
		Dialogo_control_ ( 	wxWindow* ventanaPadre, wxWindowID id=ID_DIALOGO__control_,
							const wxString& titulo= wxT ("Configurar _control_"),
							const wxPoint&  posicion=wxDefaultPosition,
							const wxSize&   tam=wxDefaultSize,
							long  			estilo=wxCAPTION | 
													wxRESIZE_BORDER |
													wxSYSTEM_MENU );
		
		void ConstruirControles();
		void Init ();
		
		/* Metodos getter y setter */
		

		
		/*	Lee el nombre y valores del elemento y los representa en
		el cuadro de dilogo */
		virtual void getPropiedades ( Elemento* elemento );
	
		/*	Hace que un elemento adquiera las propiedades del dilogo */
		virtual void setPropiedades ( Elemento* elemento );

		/* 	Gestores de eventos */
		
	private:
		
		/*	Estos nombres de variable no son del todo representativos,
			ya que el usuario puede invertir los valores y escribir que
			por ejemplo 0V es el voltaje que el _control_ emite cuando se
			pulsa. Se usan estos nombres porque por defecto se usaran
			0V para el reposo y "false" para el estado no-conectado*/
		bool 	conexionReposo		;
		bool 	conexionActivacion	;	
		int	voltajeReposo		;
		int	voltajeActivacion	;
		char	nombreElemento[MAX_LONG_NOMBRE_ELEMENTO];
		
		
		/*	Los controles. */
		
		/* 	Los contenedores: Hay un contenedor principal que apila todos
			los demas controles en vertical de la siguiente forma
				1.- El texto descriptivo 
				2.- El contenedor central que contiene
					2.0.-El contenedor para el nombre
					2.1- El contenedor para los datos del _control_ en reposo
					2.2- El contenedor para los datos del _control_ activado
				3.- El contenedor de los botones Aceptar Cancelar*/
		wxBoxSizer* contenedorPrincipal;
		wxBoxSizer*	contenedorCentral;
			wxBoxSizer*	contenedorNombre;
			wxBoxSizer*	contenedorReposo;
			wxBoxSizer* contenedorActivacion;
		wxBoxSizer*	contenedorBotones;
		
		wxStaticText* lblTextoDescriptivo ;
		
		
		wxStaticText* lblNombre;
		wxStaticText* lblConexionReposo;
		wxStaticText* lblConexionActiva;
		wxStaticText* lblVoltajeReposo;
		wxStaticText* lblVoltajeActivacion;
		
		wxTextCtrl*	txtNombre;
		wxChoice*	chcConexionReposo;
		wxChoice*	chcConexionActivacion;
		
		wxSpinCtrl*	spnVReposo;
		wxSpinCtrl* spnVActivacion;
		/*	Este es el estilo que van a tener los botones. Por defecto
			no tienen ningun estilo especial, aunque esto se puede cambiar
			asignando otros valores en lugar de 0, como las constantes
			wxSUNKEN_BORDEN, etc... Consultar los estilos de ventana en
			la documentacion de wx */
		#define ESTILO	0
		
		/*	Cantidad de pixeles dibujados alrededor de los botones*/
		#define TAM_BORDE 5
		
		#define VOLTAJE_MINIMO 			0
		#define VOLTAJE_MAXIMO			50
		#define VOLTAJE_EN_REPOSO		0
		#define VOLTAJE_EN_ACTIVACION	9
		wxButton*	botonAceptar;
		wxButton*	botonCancelar;
};

#endif
