#include "dialogo.h"


IMPLEMENT_CLASS (Dialogo, wxDialog)
Dialogo::Dialogo()
{
	Init();
}

Dialogo::Dialogo ( 	wxWindow* ventanaPadre, wxWindowID id,
							const wxString& titulo,
							const wxPoint&  posicion,
							const wxSize&   tam,
							long  			estilo ) 
{
		Init();
		Dialogo::Create (ventanaPadre, id, titulo, posicion, tam, estilo);
}

		
void Dialogo::Init()
{
	/* Inicializar variables */
}


bool Dialogo::Create (		wxWindow* ventanaPadre, wxWindowID id,
							const wxString& titulo,
							const wxPoint&  posicion,
							const wxSize&   tam,
							long  			estilo )
{
//	wxLogMessage ("Invocado Dialogo::Create()");
	SetExtraStyle (wxWS_EX_BLOCK_EVENTS | wxDIALOG_EX_CONTEXTHELP);	
	
	if (!wxDialog::Create (ventanaPadre, id, titulo, posicion,tam,  estilo))
		return false;
	ConstruirControles();
	GetSizer()->Fit (this);
	Centre();
	return true;

}



void Dialogo::ConstruirCuadroDeDialogo (wxSizer* contenedorCentral)
{
	contenedorPrincipal	=	new wxBoxSizer 	(wxVERTICAL);
	
	
	
	/* Primera "fila". Solo hay un texto descriptivo*/
	lblTextoDescriptivo = new wxStaticText (this, wxID_STATIC, 
		wxT("Introduzca los valores de configuracin para este elemento"),
		wxDefaultPosition, wxDefaultSize, ESTILO);
	
	/* Segunda "fila". Hay un texto y un control para escribir el nombre*/
	contenedorNombre		=	new wxBoxSizer	 (wxHORIZONTAL);
	lblNombre = new wxStaticText (this, wxID_STATIC, 
		wxT("Nombre del elemento (optativo)"),
		wxDefaultPosition, wxDefaultSize, ESTILO);
	contenedorNombre->Add (lblNombre,
		wxSizerFlags(1).Center().Left().Border (wxALL, 5) );
	
	/* El nombre de los elementos se ver validado por un validador
		Se impide todo texto que no sea alfanumrico*/
	txtNombre = new wxTextCtrl (this, ID_NOMBRE, wxT(""), 
		wxDefaultPosition, wxDefaultSize, ESTILO,
		wxTextValidator(wxFILTER_ALPHANUMERIC, &textoNombre));
	
	contenedorNombre->Add (txtNombre,
		wxSizerFlags(1).Left().Border (wxALL, 5) );
	
			
		
	/* Se crea la ultima fila con los botones Aceptar y Cancelar */
	contenedorBotones	=	new wxBoxSizer	(wxHORIZONTAL);
	botonAceptar = new wxButton (this, wxID_OK, wxT("Aceptar"), 
							wxDefaultPosition, wxDefaultSize, ESTILO);
	botonCancelar = new wxButton (this, wxID_CANCEL, wxT("Cancelar"), 
							wxDefaultPosition, wxDefaultSize, ESTILO);						
	contenedorBotones->Add ( botonAceptar, 
				wxSizerFlags(1).Center().Border(wxALL, TAM_BORDE) );
	contenedorBotones->Add ( botonCancelar, 
				wxSizerFlags(1).Center().Border(wxALL, TAM_BORDE) );
				
				
	/*	Se apilan los controles*/
	contenedorPrincipal->Add (lblTextoDescriptivo,	
				wxSizerFlags(1).Left().Border(wxALL, TAM_BORDE));
	contenedorPrincipal->Add (contenedorNombre,
		wxSizerFlags(1).Left().Border (wxALL, 5) );
	contenedorPrincipal->Add (contenedorCentral, 
				wxSizerFlags(3).Left().Expand() );
	contenedorPrincipal->Add (contenedorBotones,
				wxSizerFlags(1).Center() );
	
	
	/*	Se establece el contenedorPrincipal como contenedor de esta ventana*/
    this->SetSizer (contenedorPrincipal);
	
}


void Dialogo::TransferirDeSpinAVoltaje (wxSpinCtrl* spin, Voltaje* voltaje)
{
			assert (spin!=NULL);
			assert (voltaje!=NULL);
			int valor = spin->GetValue();
			voltaje->setValor(valor);
}

void Dialogo::TransferirDeVoltajeASpin (wxSpinCtrl* spin, Voltaje* voltaje)
{
	assert (spin!=NULL);
	assert (voltaje!=NULL);
	
	spin->SetValue 	( (int)	voltaje->getValor() );
}


void Dialogo::TransferirDeControlANombreDeElemento (Elemento* e)
{
	assert (e!=NULL);
	textoNombre = txtNombre->GetValue();
	e->setNombreElemento (textoNombre);
}

void Dialogo::TransferirDeNombreDeElementoAControl (Elemento* e)
{
	assert (e!=NULL);
	textoNombre = e->getNombreElemento();
	txtNombre->SetValue ( textoNombre );
}

void Dialogo::TransferirDeChoiceAConexion (wxChoice* chc, Conexion* c)
{
	assert (c!=NULL);
	wxString cadena = chc->GetStringSelection();
	int codigo = cadena.Cmp ("activado");
	if (codigo==0)
	{
		c->setValor	(true);
	}
	else {
		c->setValor (false);
	}
}

void Dialogo::TransferirDeConexionAChoice (wxChoice* chc, Conexion* c)
{
	assert (c!=NULL);
	bool activacion = c->getValor();
	if (activacion)
	{
		chc->SetStringSelection ("activado");
	}
	else {
		chc->SetStringSelection ("desactivado");
	}
}


void Dialogo::TransferirDeControlAGiro	(wxChoice* chc, Giro* giro)
{
	assert (giro!=NULL);
	wxString seleccion = chc->GetStringSelection();
	
	wxString stop	=	wxString("se detiene");
	wxString cw		=	wxString("gira a la derecha");
	wxString ccw	=	wxString("gira a la izquierda");
	int codigo;
	
	codigo = seleccion.Cmp(stop);
	if (codigo == 0) {
		giro->setValor("stop");
		return ;
	}
	
	codigo = seleccion.Cmp(cw);
	if (codigo == 0) {
		giro->setValor("cw");
		return ;
	}
	codigo = seleccion.Cmp(ccw);
	if (codigo == 0) {
		giro->setValor("ccw");
		return ;
	}
	
}
	
void Dialogo::TransferirDeGiroAControl	(wxChoice* chc, Giro* giro)
{
	assert (giro!=NULL);
	char stop[]	=	"stop";	
	char cw[]	=	"cw";
	char ccw[]	=	"ccw";
	
	wxString wxstop = 	wxString ("se detiene");
	wxString wxcw	= 	wxString ("gira a la derecha");	
	wxString wxccw	= 	wxString ("gira a la izquierda");
	int codigo;
	
	codigo = strcmp (giro->toString(), stop);
	if (codigo==0) {
		chc->SetStringSelection (wxstop);
	}
	codigo = strcmp (giro->toString(), cw);
	if (codigo==0) {
		chc->SetStringSelection (wxcw);
	}
	codigo = strcmp (giro->toString(), ccw);
	if (codigo==0) {
		chc->SetStringSelection (wxccw);
	}
	
	
}

void Dialogo::TransferirDeChoiceAMagnetismo (wxChoice* chc, Magnetismo* m)
{
	assert (m!=NULL);
	wxString cadena = chc->GetStringSelection();
	int codigo = cadena.Cmp ("activado");
	if (codigo==0)
	{
		m->setValor	(true);
	}
	else {
		m->setValor (false);
	}
}

void Dialogo::TransferirDeMagnetismoAChoice (wxChoice* chc, Magnetismo* m)
{
	assert (m!=NULL);
	bool activacion = m->getValor();
	
	if (activacion)
	{
		chc->SetStringSelection ("activado");
	}
	else {
		chc->SetStringSelection ("desactivado");
	}
}

void Dialogo::TransferirDeSpinAFrecuencia (wxSpinCtrl* spin, Frecuencia* frecuencia)
{
			assert (frecuencia!=NULL);
			int valor = spin->GetValue();
			frecuencia->setValor(valor);
}

void Dialogo::TransferirDeFrecuenciaASpin (wxSpinCtrl* spin, Frecuencia* frecuencia)
{
	assert (frecuencia!=NULL);
	spin->SetValue 	( (int)	frecuencia->getValor() );
}

void Dialogo::TransferirDeSpinAColor (wxSpinCtrl* spinR, wxSpinCtrl* spinG,
							wxSpinCtrl* spinB, Luminosidad* color)
{
	assert (color!=NULL);
	
	int r, g, b;
	char nuevoColor[6];
	
	/*	Se recuperan los tres componentes de color */
	r= spinR->GetValue();
	g= spinG->GetValue();
	b= spinB->GetValue();
	
	/*	El color es una secuencia de tres hexadecimales, y se aaden
		ceros por la izquierda si es necesario */
	sprintf (nuevoColor, "%02x%02x%02x", r, g, b);
//	wxLogMessage ("El color es %s", nuevoColor);
	color->setValor (nuevoColor);
//	wxLogMessage ("El color almacenado es %s", color->toString());
}							
						
void Dialogo::TransferirDeColorASpin (wxSpinCtrl* spinR, wxSpinCtrl* spinG,
							wxSpinCtrl* spinB, Luminosidad* color)
{
	int r, g, b;
	
	assert (color!=NULL);
	/*	Se pasa el color a una cadena y se recuperan los hexadecimales
		que haya dentro */
//	wxLogMessage ("El color almacenado es %s", color->toString());
	sscanf (color->toString(), "%02x%02x%02x", &r, &g, &b);
//	wxLogMessage ("R, G y B valen %i, %i y %i", r, g, b);
	spinR->SetValue(r);
	spinG->SetValue(g);
	spinB->SetValue(b);
}						
