#include "pagina.h"

/* 	Este constructor es el constructor estndar por defecto. 
	Es un recubrimiento del otro constructor, que es el fundamental, aunque
	solo es llamado cuando alguien quiere abrir un archivo */
Pagina::Pagina (wxNotebook* padre) : wxPanel ( padre )
{
	assert (padre!=NULL);
	SetValoresIniciales();
 	sistema=new Sistema();
}

/* 	En este constructor se hacen las tareas fundamentales */
Pagina::Pagina (Sistema* sistemaPasado, wxNotebook* padre) : wxPanel ( padre )
{
	assert (sistemaPasado!=NULL);
	wxLogMessage ("----Pagina::Pagina(sistemaPasado, padre)----");
	wxLogMessage ("Fichero %s, linea %i", __FILE__, __LINE__);
	sistema= sistemaPasado;
	SetValoresIniciales();	
}
Pagina::~Pagina()
{
}
ElementoVisual *Pagina::insertarElemento (int tipoDeElemento, int x, int y)
{
	assert (x>=0);
	assert (y>=0);
	
	wxLogMessage ("----Pagina::insertarElemento----");
	wxLogMessage ("Fichero: %s, linea %i", __FILE__, __LINE__);
	

	/*	En funcion del elemento que se haya activado en la barra
		de herramientas se inserta un determinado elemento */
			
	ElementoVisual* elemento ;
	
	elemento=FabricaDeElementos (tipoDeElemento);
	wxLogMessage ("Fabricado el elemento");
	elemento->SetXY(x, y);
	wxLogMessage ("Aadiendo elemento visual...");
	/*	En el futuro esto tal vez sea un sistema visual.*/
	int num_elementos=sistema->anadirElemento (elemento);
	wxLogMessage ("...aadido elemento visual!");
	assert (num_elementos>=1);
	/*	Si hay demasiados elementos en pantalla se avisa */
	if (num_elementos>=MARGEN_AVISO)
	{
		wxString cadenaAviso(CADENA_AVISO);
		wxMessageBox(cadenaAviso);
	}
	wxLogMessage ("Hay %i elementos\n", num_elementos);
	wxLog::FlushActive();
	elemento->SetPadre (sistema);

	ReiniciarEstado(); 
	
//	wxMessageBox ("Dibujo con exito");

	return elemento;
		
}

void Pagina::ReiniciarEstado()
{
	wxLogMessage ("----Pagina::ReiniciarEstado----");
	wxLogMessage ("Fichero %s, linea %i", __FILE__, __LINE__);
	

	/*	Si haba algo seleccionado se deselecciona */
	if ( elemento_emisor != NULL )
	{
		elemento_emisor->Deseleccionar();
	}
	elemento_emisor = elemento_receptor = NULL;
	estado_actual	= INICIAL ;
	wxClientDC contexto(this);
	contexto.Clear();
	Dibujar (contexto);
}
void Pagina::Dibujar(wxDC& contexto)
{
	wxLogMessage ("----Pagina::Dibujar----");
//	wxLogMessage ("Fichero %s, linea %i", __FILE__, __LINE__);
	/* 	Se obtiene un iterador al principio */
	map <string, Elemento*>::iterator i;
	map <string, Elemento*> elementos;
	assert (sistema!=NULL);
	elementos = sistema->getElementos();
//	int tam = elementos.size();
//	wxLogMessage ("Recuperados %i elementos", tam);
	for ( 	i = elementos.begin();
			i!= elementos.end();
			i++)
	{
		ElementoVisual* elemento=dynamic_cast<ElementoVisual*>(i->second);
		
		if (elemento->EsVisible()) {
			elemento->DibujarElemento (contexto);
			elemento->DibujarRelaciones (contexto);
		}
		else {
//			wxLogMessage (elemento->GetNombre()+" no es visible");
		}
	}
//	wxLogMessage ("Se ha redibujado la pagina\n");	
	
}

void Pagina::BorrarElementoSistemaSiExiste (Elemento* sistemaDentroDePagina)
{
	/* 	Nos han pasado un elemento sistema. Este elemento sistema debera
		ir dentro de un sistema visual por lo que recorremos los elementos
		y buscamos dentro de los elementos envueltos. Si encontramos alguno
		lo eliminamos y terminamos.
		
		En realidad este mtodo podramos usarlo para muchas cosas ms, pero
		no lo hacemos porque no ha sido necesario */
		
	wxLogMessage ("\n---Pagina::BorrarElementoSistemaSiExiste---");
	const char* nombre=sistemaDentroDePagina->getNombreElemento();
	wxLogMessage ("Borrando %s", nombre);
	map <string, Elemento*>::iterator i;
	map <string, Elemento*> elementos;
	elementos = sistema->getElementos();
	for ( 	i = elementos.begin();
			i!= elementos.end();
			i++)
	{
		ElementoVisual* elemento=dynamic_cast<ElementoVisual*>(i->second);
		const char* nombre = elemento->getNombreElemento();
		wxLogMessage ("Examinando %s", nombre);
		Elemento* elementoEnvuelto = elemento->getElementoEnvuelto();
		/* Si el elemento envuelto es el que hay que borrar...*/
		if (elementoEnvuelto==sistemaDentroDePagina)
		{
			wxLogMessage ("Finalizando destruccin");
			/*	Hack: Me aprovecho del mtodo eliminarseleccionado
				que borra el elemento llamado elemento_emisor
				si estamos en el estado seleccionado */
			//elemento_emisor=elemento;
//			estado_actual = EN_ELEM_SELECCIONADO;
//			EliminarSeleccionado();
			sistema->eliminarElemento (elemento);
			return ;
		}
	}
	return ;
}
wxString Pagina::getXML()
{
	wxLogMessage ("Obteniendo XML");
	return wxString (sistema->getCadenaXML()) ;
} /* Fin de getXML*/

Sistema* Pagina::GetSistema()
{
	return sistema;
}

void Pagina::SetSistema(Sistema* s)
{
	//Se borra el sistema anterior
	if (sistema!=NULL)
	{
		delete sistema;
		sistema=NULL;
	}
	sistema = s;
}
ElementoVisual* Pagina::FabricaDeElementos (int tipoDeElemento)
{
	wxLogMessage ("----Pagina::FabricaDeElementos----");
	wxLogMessage ("Fichero %s, linea %i", __FILE__, __LINE__);
	
	/* 	Este mtodo antes era un enorme switch que se 
		traslad a la clase FabricaDeElementosVisuales */
		
	/* Se crea el elemento..*/
	ElementoVisual* elementoADevolver=NULL;
	FabricaDeElementosVisuales fabrica;
	elementoADevolver=fabrica.CrearElementoVisual(tipoDeElemento, this);
	
	/* ..se comprueban un par de cosas...*/
	bool elemento_es_null = (elementoADevolver==NULL);
	assert (!elemento_es_null);
	wxLogMessage ("Se van a poner los sufijos...");
	/*...se actualizan ciertos detalles...*/
	elementoADevolver->SetSufijo (numElementos[tipoDeElemento]);
	/*	Se actualiza el  numero de elementos construidos */
	numElementos[tipoDeElemento]++;
	
	wxLogMessage ("---Fin de Pagina::FabricaDeElementos---");
	/*... y se termina*/
	return elementoADevolver;
	
}



void Pagina::EliminarSeleccionado()
{
//	wxLogMessage ("Eliminando, estado %i", estado_actual);
	if (estado_actual == EN_ELEM_SELECCIONADO)
	{
		wxLogMessage ("Borrando elemento");
		/* 	El metodo OnRaton habr puesto por nosotros el elemento
			emisor apuntando a la figura seleccionada. La destruccin
			tiene que hacerse dando estos pasos */
		Elemento* elemento_a_destruir = elemento_emisor;
		
		/*Se va a crear un dialogo que pregunte al usuario si esta seguro */
		wxString mensaje="Desea eliminar el elemento ";
		mensaje+=wxString(elemento_a_destruir->getNombreElemento());
		mensaje+="?";
		wxMessageDialog dialogo (this, mensaje, "Eliminar elemento", 
					wxOK|wxCANCEL,wxDefaultPosition);
		if (dialogo.ShowModal()==wxID_OK)
		{
			assert (elemento_a_destruir!=NULL);
			sistema->eliminarElemento (elemento_a_destruir);
		}

	//	wxLogMessage ("Listo para borrar emisor");
//		delete elemento_emisor;
//		elemento_emisor=NULL;
//		wxLogMessage ("Emisor borrado");
		ReiniciarEstado();
	} /* Fin de comprobar si hay algn elemento seleccionado */
} /* Fin de EliminarSeleccionado*/

void Pagina::ConmutarSimulacion (bool modo)
{
	modoSimulacion=modo;
}


/*	Elimina un temporizador del vector correspondiente*/
void Pagina::BorrarTemporizador(TemporizadorVisual* pasado)
{
	vector<TemporizadorVisual*>::iterator i;
	for (	i=temporizadores.begin();
			i!=temporizadores.end();
			i++ )
	{
		TemporizadorVisual* examinado=*i;
		if ( pasado == examinado )
		{
			temporizadores.erase(i);
		}
	} /* Fin del for */
} /* Fin de borrar temporizador */

void Pagina::DetenerTemporizadores ()
{
	vector<TemporizadorVisual*>::iterator i;
	for (	i=temporizadores.begin();
			i!=temporizadores.end();
			i++ )
	{
		TemporizadorVisual* temporizador_examinado = *i;
		temporizador_examinado->Stop();
	} /* fin del for */
} /* Fin de detener temporizadores */
//#define DEBUG_ON_RATON


void Pagina::Actualizar()
{
	wxClientDC 	contexto(this);
	wxSize 		tamano_pagina = this->GetSize();
	wxBufferedDC contexto_con_buffer (&contexto, tamano_pagina, wxBUFFER_CLIENT_AREA);
	contexto_con_buffer.Clear();
	Dibujar (contexto_con_buffer);
}



ElementoVisual* Pagina::BuscarElemento (wxPoint punto)
{
	map <string, Elemento*>::iterator i;
	map <string, Elemento*> elementos;
	elementos = sistema->getElementos();
	
	wxRect zona_visual_del_elemento;
	
	/* 	Para cada elemento se comprueba el rectangulo determinado
		por su coordenada (x, y) y el rectangulo 
		en (X+ANCHURAICONOS, Y+ANCHURA_ICONOS).*/
	for (	i=elementos.begin();
			i!=elementos.end();
			i++ )
	{
		ElementoVisual* elemento=dynamic_cast<ElementoVisual*>(i->second);
		
		zona_visual_del_elemento = elemento->GetRectangulo();
		
		/* 	Si el punto donde hemos hecho click est dentro de ese
			rectangulo, es que hemos hecho click en el elemento */
			
		if (zona_visual_del_elemento.Inside (punto))
		{
			return elemento;
		} /* Fin del if */

	} /*Fin del for */
	
	/*	Si no se ha encontrado una figura devolvemos NULL */
	return NULL;
}


BEGIN_EVENT_TABLE(Pagina, wxPanel)	
	EVT_MOUSE_EVENTS	(Pagina::OnRaton)
	EVT_PAINT 			(Pagina::OnPaint)
END_EVENT_TABLE()


