#ifndef _pagina_h
#define _pagina_h


#include "sistema.h"
#include "pulsador.h"
#include "rele.h"
#include "biestable.h"
#include "zumbador.h"
#include "electroiman.h"
#include "contador.h"
#include "reed.h"

#include "elementosvisuales/pulsadorvisual.h"
#include "elementosvisuales/lamparavisual.h"
#include "elementosvisuales/electroimanvisual.h"
#include "elementosvisuales/biestablevisual.h"
#include "elementosvisuales/relevisual.h"
#include "elementosvisuales/reedvisual.h"
#include "elementosvisuales/zumbadorvisual.h"
#include "elementosvisuales/contadorvisual.h"
#include "elementosvisuales/temporizadorvisual.h"
#include "elementosvisuales/fotosensorvisual.h"
#include "elementosvisuales/sistemavisual.h"
#include "elementosvisuales/motorvisual.h"
#include "elementosvisuales/fabricaelementosvisuales.h"
#include "eventoclicknotebook.h"
#include "constantes.h"


#include "wx/image.h"
#include "wx/wx.h"
#include "wx/generic/dragimgg.h"



#include <vector>
class Pagina : public wxPanel
{
	public:
		Pagina (wxNotebook* padre);
		Pagina (Sistema* sistemaPasado, wxNotebook* padre);
		~Pagina();
		ElementoVisual* insertarElemento (int tipoDeElemento, int x, int y);
		
		/*	Esto permite que el usuario interrumpa de forma limpia
			acciones que ha dejado a medias */
		void 			ReiniciarEstado();
		/*	Dibuja (o mejor dicho redibuja) los elementos que hay en 
			el sistema de la pgina actual */
		void			Dibujar(wxDC& contexto);
		
		/*	Este mtodo invoca al destructor de un elemento
			si existe alguna referencia a l. Lo usa el
			mtodo Ventana::OnMenuEdicionEliminarSistema*/
		void BorrarElementoSistemaSiExiste (Elemento* sistemaDentroDePagina);
		
		/*	Devuelve una cadena con el XML que hay dentro */
		wxString		getXML();
		
		/*	Devuelve una referencia al sistema que esta pgina manipula */
		Sistema* 	GetSistema();
		void		SetSistema(Sistema* s);
		/*	Esto inserta unos cuantos elementos y comprueba si un cambio en
			la entrada produce un cambio en la salida */
		void TestInsercion ();
		
		/* Inserta un sistema pequeo compuesto por un pulsador
			cuya salida va a un temporizador y la salida de este
			a una lampara. Cuando se pulsa el pulsador se
			activa el temporizador, que tras un cierto tiempo 
			encender la lampara*/
		void TestTemporizador();
		
		void TestSemaforo();		
		/*	Esto tal vez se podra haber hecho de otra forma, probablemente
			con un factory method mejor, pero habra "problemas mu gordos" de
			solventar y qu quieres que te diga?, pues que estoy
			un poco harto de pensar. Si no te gusta, hzlo mejor. Eso s
			a ver que haces si al usuario le da por pulsar muchas veces el
			mismo boton, machote. Avisado quedas */
		ElementoVisual*	FabricaDeElementos (int tipoDeElemento);
		/*	Eliminar el elemento que estuviera seleccionado
			si lo haba */
		void EliminarSeleccionado();
		
		void ConmutarSimulacion (bool modo);
		
		/*	Detiene todos los temporizadores que pudiera haber
			en esta pgina */
		void DetenerTemporizadores ();
		
		/*	Fuerza un repintado de la pagina */
		void Actualizar();
	private:
		/*	Nos dice si estamos construyendo o simulando */
		bool modoSimulacion;
		/*	Nos dice cuantos pulsadores hay, cuantos electroimanes, etc...
			Se usa solo para poner nombres a los elementos, p.ej, Pulsador 1,
			Electroimn 2...*/
		int numElementos[TOTAL_ELEMENTOS];
		
		/* Mtodo auxiliar utilizado por el constructor para
			poner los valores iniciales en una nueva pagina*/
		void SetValoresIniciales();
		
		/* Comprueba si es un puntero NULL*/
		void ComprobarPuntero (void* );
		/*	Nos dice si hay algn elemento en la posicin pasada */
		ElementoVisual* BuscarElemento (wxPoint punto);
		
		/*	Cada pagina conoce al subsistema que dibuja en 
			la pantalla */
		Sistema*	sistema;
		
		/*	Este metodo comprueba si los dos elementos estn en el mismo
			sistema (en cuyo caso la conexin es trivial) o si son elementos
			que estn en distintos sistemas, en cuyo caso hay que hacer 
			las conexiones de forma un poco ms compleja (ver implementacin */
		void ConectarElementos (ElementoVisual* primario, ElementoVisual* secundario,
			int pos_salida_del_emisor, int pos_entrada_del_receptor);
			
		/*	Este metodo comprueba si los elementos se pueden relacionar */
		bool SonElementosConectables (ElementoVisual* primario, 
			ElementoVisual* secundario);
			
		/*	Se usa para saber el estado de un elemento, asi se puede saber 
			si estamos esperando que el usuario arrastre un
			elemento o suelte el botn del ratn */
		enum estado {
			INICIAL, EN_ESPERA, EN_MOVIMIENTO, EN_ELEM_SELECCIONADO
		};
		estado estado_actual;
		/*	Figura que se est arrastrando */
		ElementoVisual* figura_en_arrastre;
		
		/*	Esto se usa cuando el usuario quiere
			conectar la salida de elemento_emisor a la
			entrada de elemento_receptor */
		ElementoVisual* elemento_emisor;
		ElementoVisual* elemento_receptor;
		
		int pos_salida_emisora;
		int pos_entrada_receptora;
		
		/*	Posicin donde se inicia el arrastre */
		wxPoint			pos_inicio_del_arrastre;
		
		/*	Todos los temporizadores que se crean se almacenan
			aqu tambin, para poder pararlos si es necesario*/
		vector<TemporizadorVisual*> temporizadores;
		/*	Esta es la imagen que se muestra cuando el usuario
			arrastra un control para modificar su posicin.
			Podra ir codificada en los ElementosVisuales y tal
			vez permitir que fuera bitmaps diferentes, pero por 
			simplicidad se deja as. Por defecto, las imagenes
			que se muestran son las mismas de los elementos */
		wxGenericDragImage*		imagen_de_arrastre;
		
		
		/*	Elimina un temporizador del vector correspondiente*/
		void BorrarTemporizador(TemporizadorVisual* t);
		
		/*	Es importante aadir un gestor de
			eventos que controle la pulsacion de
			botones del raton en las paginas del sistema
			ya que estos eventos NO SE PASAN A LOS CONTROLES
			CONTENEDORES de esta pagina. En otras palabras
			hay que capturar el evento aqui y pasarlo
			a la clase contenedora */
		void OnBotonPulsado 	( wxPoint punto_del_click );
		void OnDobleClick		( wxPoint punto_del_doble_click );
		
		/*	Si se pulsa el raton y se esta en el modo simulacion 
			se dispara este evento */
		void ProcesarPulsacion	( wxMouseEvent& evento );
		/*	Esto procesa todos los eventos del raton */
		void OnRaton			( wxMouseEvent& evento );
		void OnPaint			( wxPaintEvent& evento );
		
		
		DECLARE_EVENT_TABLE()
};
#endif
