#include "ventana.h"

/*	En este fichero se implementan los mens. Se hace as para que haya
	mas coherencia, sea ms fcil de mantener y para que sea ms
	fcil navegar por los archivos */

void Ventana::OnMenuArchivoAbrir (wxCommandEvent& evento)
{
	wxLogMessage("-----Ventana::OnMenuArchivoAbrir-----");		
	wxLogMessage("Archivo %s, linea %i", __FILE__, __LINE__);		
	/*	Mostramos un cuadro de dilogo de apertura de ficheros...*/
	wxString titulo 	= wxT ("Abrir archivo...");
	wxString opciones	= wxT ( "Ficheros XML (*.xml)|*.xml" );
	wxString directorio = wxT (".");
	wxString archivo	= wxEmptyString;
	
	wxFileDialog dialogo (this, titulo, directorio, archivo, 
		opciones, wxOPEN);
		
	if ( dialogo.ShowModal() == wxID_OK )
	{
		wxString ruta = dialogo.GetPath ();
		Parser* parser= new Parser();
		bool esValido = parser->esValido (ruta.c_str());
		if (esValido)
		{
			wxLogMessage ("Archivo %s correcto", ruta.c_str());
			/*Aqui se cerrarian todas las paginas y se abre una nueva
				con ese sistema */
			for (unsigned int i=0; i<libro->GetPageCount(); i++)
			{
				Pagina* p= (Pagina*) libro->GetPage(i);
				delete p;
			}
			/* 	Se crea la pagina donde se mostraran los elementos
				del sistema parseado */
			Pagina* paginaSistemaParseado=new Pagina (libro);		
			
			/* 	Se recuperan los elementos desde el XML. Esos
				elementos necesitan saber en qu pagina se van a pintar*/
			Sistema* sistemaParseado = RecuperarSistemaDesdeXML 
				(parser, paginaSistemaParseado);
			wxLogMessage ("Sistema Recuperado desde archivo");
			paginaSistemaParseado->SetSistema (sistemaParseado);
			wxString nombrePagina(sistemaParseado->getNombreElemento());
			libro->AddPage (paginaSistemaParseado, nombrePagina, false);
			this->sistemaActivo = sistemaParseado;			
		}
		else {
			if (parser->haHabidoErrores())
			{
				char* error = parser->obtenerError();
				wxLogMessage ("-->Error de analisis");
				wxLogMessage (error);
			}
		}
	}
} /*Fin de OnMenuArchivoAbrir*/


wxString Ventana::ObtenerNombreDeFichero()
{
	wxString titulo = wxT("Guardar como...");
	wxString extensiones =
	   wxT("Ficheros XML (*.xml)|*.xml");
	wxString directorio(".");
	wxString nombrePorDefecto = wxEmptyString;
	
	wxFileDialog dialogo (this, titulo, directorio, 
		nombrePorDefecto, extensiones, wxSAVE);
	
	if (dialogo.ShowModal() == wxID_OK)
	{
		return dialogo.GetPath();
	}
	else return "";
}
void Ventana::Guardar( wxString& nombre )
{
	wxString xml;
	
	/*	Se comprueba si hay alguna pgina activa */
	Pagina *p = (Pagina*) libro->GetCurrentPage();
	if (p!=NULL)
	{	
		xml = p->getXML();
	}
	else {
		/*	Si no hay pginas, se sale. Es lo ms coherente para el usuario*/
		return;
	 } /*Fin del else */
	
	/*	Si no hay nombre, se pide */
	if (nombre=="")
	{
		/*	Este nombre es el nombre de fichero que maneja la ventana de
			la aplicacin */
		nombreArchivo = ObtenerNombreDeFichero();
	}
	
	/*	Si el usuario sigue sin haber puesto un nombre el mtodo
		termina sin ms */
	if (nombreArchivo=="")
	{
		return ;
	}
	
	/*	En otro caso, se almacena todo*/
	wxFile fichero (nombreArchivo, wxFile::write);
	if (!fichero.IsOpened())
	{
		wxLogMessage (wxString("No se pudo abrir el fichero indicado:")+nombreArchivo);
	}
	const void* datos = (void*)(xml.c_str());
	int longitudDatos = xml.Len();
	int numbytes=fichero.Write (datos, longitudDatos);
	wxLogMessage ("Almacenados %i bytes", numbytes);
	fichero.Close();
	
	/*	El titulo de la ventana queda mejor as */
	this->SetTitle (wxString("SimProc - ") + nombreArchivo);
	return ;
} /* Fin de guardar */
void Ventana::OnMenuArchivoGuardar 		( wxCommandEvent& evento )
{
	/*	Se fuerza el reguardado con el nombre que hubiera. 
		Si no lo hay, el mtodo Guardar lo pedir */
	Guardar(nombreArchivo);
} /*Fin de OnMenuArchivoGuardar */

void Ventana::OnMenuArchivoGuardarComo 	( wxCommandEvent& evento )
{
	wxString nombreVacio("");
	/*	Con esto se fuerza al mtodo Guardar para que se le pida
		al usuario un nombre de fichero */
	Guardar(nombreVacio);
}/*Fin de OnMenuArchivoGuardarComo*/

void Ventana::OnMenuEdicionEliminarElemento	( wxCommandEvent& evento )
{
	Pagina *p = (Pagina*) libro->GetCurrentPage();
	if (p!=NULL)
	{	
		p->EliminarSeleccionado();
	}
} /* Fin de OnMenuEdicionEliminarElemento */

void Ventana::OnMenuEdicionEliminarSistema( wxCommandEvent& evento )
{
	Pagina *p = (Pagina*) libro->GetCurrentPage();
	if (p!=NULL)
	{	
		/* Se toma nota del sistema que almacena esta pgina */
		Sistema* sistemaDentroDePagina=p->GetSistema();
		/*...y se destruye la pagina */
		
		int numPagina = libro->GetSelection();
		libro->DeletePage(numPagina);
		/*	Despues de borrar una pagina hay que asegurarse de que
			la pagina borrada no es la ultima. Si es la ultima, hay que
			hacer que sistemaActivo apunte a NULL */
		if (sistemaDentroDePagina==sistemaActivo)
		{
			sistemaActivo=NULL;
		}
		/* 	Se pide a todas las pginas que, si lo contienen, 
			borren el sistema visual cuyo sistema envuelto sea el mismo
			que este sistema*/
		for (unsigned int i=0; i<libro->GetPageCount(); i++)
		{
			Pagina* pagina= (Pagina*) libro->GetPage(i);
			pagina->BorrarElementoSistemaSiExiste (sistemaDentroDePagina);
		}
	}
} /* Fin de OnMenuEdicionEliminarSistema */

void Ventana::OnMenuInsertarSistemaDePrueba(wxCommandEvent& evento)
{
	Pagina* p=new Pagina(libro);
//	wxMessageBox("Aqui se deberia insertar un sistema");
	libro->AddPage(p, wxString ("Sistema de prueba"), false);
	libro->SetSelection(1);
	p->TestInsercion();
} /*Fin de OnMenuInsertarSistemaDePrueba*/

void Ventana::OnMenuInsertarSistemaDePrueba2(wxCommandEvent& evento)
{
	Pagina* p=new Pagina(libro);
//	wxMessageBox("Aqui se deberia insertar un sistema");
	libro->AddPage(p, wxString ("Sistema temporizado"), false);
	libro->SetSelection(1);
	p->TestTemporizador();
} /*Fin de OnMenuInsertarSistemaDePrueba2*/

void Ventana::OnMenuInsertarSistemaDePrueba3(wxCommandEvent& evento)
{
	Pagina* p=new Pagina(libro);
//	wxMessageBox("Aqui se deberia insertar un sistema");
	libro->AddPage(p, wxString ("Sistema temporizado"), false);
	libro->SetSelection(1);
	p->TestSemaforo();
} /*Fin de OnMenuInsertarSistemaDePrueba*/

void Ventana::OnMenuInsertarSistema (wxCommandEvent& evento)
{
	/* No se permite hacer nada mientras se est en modo simulacion*/
	if (modoSimulacion==true) return;
	/*	Si no hay ningun sistema activo, el sistema activo es el de 
		esta primera pagina */
	if (sistemaActivo == NULL ) {
//		wxLogMessage ("Insertando sistema inicial");
		Pagina* p=new Pagina(libro);
//		wxLogMessage ("Pagina creada");
		sistemaActivo = p->GetSistema();
//		wxLogMessage ("Sistema obtenido");
		sistemaActivo->setNombreElemento("Sistema");
//		wxLogMessage ("NombreEstablecido");
		libro->AddPage(p, wxString ("Sistema"), true);
	}
	else {
		/*	El subsistema siempre se aade en la pgina activa, as
			que el sistema de la pgina recien creada se mete en la
			pgina actual */
		wxLogMessage ("No es el primer sistema");
		Pagina* pagActual = (Pagina*)libro->GetCurrentPage();
		ElementoVisual* elem= insertarObjetoEnSistema (
					SISTEMA, pagActual,
					pos_insercion_por_defecto.x, 
					pos_insercion_por_defecto.y);
		wxLogMessage ("insertado subsistema");
		SistemaVisual* sist_insertado = dynamic_cast<SistemaVisual*> (elem);
		CrearPaginaSubsistema (sist_insertado);
	}
//	ReiniciarEstadoPaginaActual();
} /*Fin de OnMenuInsertarSistema */

void Ventana::OnMenuVerXML ( wxCommandEvent& evento )
{
	wxString xml("<sistema>\n</sistema>");
	
	/*	Obtenemos el XML de los elementos que hay en la pgina activa...*/
	Pagina *p = (Pagina*) libro->GetCurrentPage();
	if (p!=NULL)
	{	
		xml = p->getXML();
	}
	else {
		/*	Si no hay pginas, se sale. Es lo ms coherente para el usuario*/
		return;
	}
	DialogoXML dialogo (this);
	dialogo.MostrarStringXML (xml);
	
	int codigo_devuelto = dialogo.ShowModal();
	
	if (codigo_devuelto == wxID_OK )
	{
//		wxLogMessage ("Pulsado el boton salir");
		return ;
	}
	if (codigo_devuelto == ID_GUARDAR)
	{
		return ;
	}
} /* Fin de OnMenuVerXML */

void Ventana::OnMenuAyudaAcercaDe( wxCommandEvent& evento )
{
	wxString mensaje(
		"SimProc. Simulacin de procesos discretos\n\n scar Gmez Garca (2006)\n"
	);
	wxString reconocimiento(
	"\nDirigido por Jess Salido Tercero.\nDepartamento de Ing. Elctrica, Electrnica y Automtica\nUniversidad de Castilla-La Mancha.\n\n"
	);
	wxMessageBox (
		mensaje+reconocimiento,	
		"Acerca de SimProc",
		wxOK | wxICON_INFORMATION);
} /*Fin de OnMenuAyudaAcercaDe(*/



void Ventana::OnMenuModoConstruccion( wxCommandEvent& evento )
{
	modoSimulacion=false;
	ConmutarSimulacion(modoSimulacion);
	wxLogMessage ("Botonera->Habilitar()");
	/* Poner esto a false permite evitar problemas si el usuario
		pulso un boton en la barra de herramientas y luego siguio trabajando
		ya que la aplicacion "recordaria" la pulsacion*/
	barraDeHerramientasEstaActiva=false;
	botonera->Habilitar();
	
	/* Se detienen los temporizadores que hubiera activos */
	for (unsigned int i=0; i<libro->GetPageCount(); i++)
	{
		Pagina* p= (Pagina*) libro->GetPage(i);
		if (p!=NULL) p->DetenerTemporizadores();
	}
}

void Ventana::OnMenuModoSimulacion	( wxCommandEvent& evento )
{
	modoSimulacion=true;
	ConmutarSimulacion(modoSimulacion);
	wxLogMessage ("Botonera->Deshabilitar");
	/* Poner esto a false permite evitar problemas si el usuario
		pulso un boton en la barra de herramientas y luego siguio trabajando
		ya que la aplicacion "recordaria" la pulsacion*/
	barraDeHerramientasEstaActiva=false;
	botonera->Deshabilitar();
	return ;
}

void Ventana::ConmutarSimulacion (bool modoSimulacion)
{
	/* Cuidado: Las paginas van de 0 a PageCount*/
	for (unsigned int i=0; i<libro->GetPageCount(); i++)
	{
		Pagina* p= (Pagina*) libro->GetPage(i);
		wxLogMessage ("Conmutando simulacion de paginas");
		if (p!=NULL) p->ConmutarSimulacion(modoSimulacion);
	}
	
	return ;
}
