/*	Este fichero contiene el metodo que recupera los elementos visuales
	que hay en un sistema escrito en XML. Lo hace colaborando con la
	clase Parser */

#include "ventana.h"
#include "fabricaelementos.h"

#include <xercesc/dom/DOM.hpp>

Sistema* Ventana::RecuperarSistemaDesdeXML(Parser* p, wxWindow* padre)
{
	
	wxLogMessage("-----Ventana::RecuperarSistemaDesdeXML-----");		
	wxLogMessage("Archivo %s, linea %i", __FILE__, __LINE__);	
	
	assert (p!=NULL);
	
	/* Nos situamos al comienzo del fichero XML*/
	DOMNode* elemento = p->inicializarRecorrido();
	assert (elemento!=NULL);
	Sistema* sistema_devuelto = ProcesarXML(elemento, padre);
	wxLogMessage ("---Fin de RecuperarSistemaDesdeXML---");
	assert (sistema_devuelto!=NULL);
	return sistema_devuelto;
}


/*	Esta es una funcion crea un elementovisual a partir de un nodo XML*/
ElementoVisual* Ventana::ObtenerElementoVisual(
	char* nombreetiquetaXML, wxWindow* pagina,
	DOMElement* elemento, Sistema* sistemaDevuelto)
{
	/* Se crean las fbricas...*/
	FabricaDeElementos 			fabricaElementosSimples;
	FabricaDeElementosVisuales  fabricaElementosVisuales;
	
	/* ...se crean los elementos... */
	Elemento* 		elementoSimple 	= 
		fabricaElementosSimples.CrearElemento (nombreetiquetaXML);
		
	ElementoVisual*	elementoVisual	= 
		fabricaElementosVisuales.CrearElementoVisual(nombreetiquetaXML, pagina);
	
	/*	...se reconstruye el objeto a partir del xml...*/
	wxLogMessage ("...construyendo elemento simple...");
	elementoSimple->construir (elemento);
	wxLogMessage ("Construido!\n...construyendo elemento visual...");
	elementoVisual->setElementoEnvuelto (elementoSimple);
	wxLogMessage ("Construido!\n...Anadiendo el sistema...");
	elementoVisual->SetPadre (sistemaDevuelto);
	
	return elementoVisual;
}


/* 	Una simple funcion auxiliar que busca el nombre de una etiqueta o de
	una etiqueta alternativa. Se usa cuando se parsea una relacin, para
	buscar un elemento que se puede llamar "primario" o "sistemaprimario"
	o quiza "secundario" vs "sistemasecundario".
	La funcin devuelve el contenido de dicha etiqueta o etiqueta alternativa.
	*/
char* obtenerNombreParticipanteEnRelacion (DOMElement* elemento_relacion, 
		const char* etiqueta, const char* etiquetaalternativa)
{
	XMLCh* etiquetaparabuscar = XMLString::transcode ( etiqueta );
	DOMNodeList* lista = elemento_relacion->getElementsByTagName ( etiquetaparabuscar );
	DOMNode *nodoNombre= lista->item(0);
	
	/* 	Si no hay un elemento <primario> es posible que haya un elemento
		llamado <sistemaprimario> */
	if (lista->getLength()==0)
	{
		etiquetaparabuscar = XMLString::transcode ( etiquetaalternativa );
		DOMNodeList* lista = elemento_relacion->getElementsByTagName 
			( etiquetaparabuscar );
		/* Si al buscar en esta alternativa no se ha hallado nada ha
			ocurrido algo grave.*/
		int tamanio_lista=lista->getLength();
		assert (tamanio_lista>0);
		
		nodoNombre=lista->item(0);
	}
	// Una vez recuperado el nombre hay que pasar al nodo hijo
	// que es donde est almacenado el nombre
	nodoNombre=nodoNombre->getFirstChild();
	
	const XMLCh* nombreRecuperado=nodoNombre->getNodeValue();
	char *nombreDelElementoBuscado=XMLString::transcode (nombreRecuperado);
	/*	Se libera la memoria utilizada temporalmente por la cadena */	
	XMLString::release ( &etiquetaparabuscar );
	
	return nombreDelElementoBuscado;
	
}
/*	Esta funcion reconoce los parmetros de las relaciones almacenadas
	en un elemento XML. Busca los nombres y los conecta de la forma
	adecuada */
void Ventana::CrearRelacion(DOMElement* elemento_relacion, Sistema* sistemaDevuelto)
{
	/* 	Estos numericos dirn la salida del elemento que va conectada
		a la entrada del otro elemento */
	int pos_entrada, pos_salida;
	/*	Requisito imprescindible */
	assert (sistemaDevuelto!=NULL);
	assert (elemento_relacion!=NULL);
	
	wxLogMessage ("---Ventana::CrearRelacion---");
	
	/* Obtenemos la posicion_de_entrada */
	const XMLCh* atributo_pos_entrada = elemento_relacion->getAttribute 
		( C ("pos_entrada"));
	pos_entrada = XMLString::parseInt (atributo_pos_entrada);
	
	const XMLCh* atributo_pos_salida = elemento_relacion->getAttribute 
		( C ("pos_salida"));
	pos_salida = XMLString::parseInt (atributo_pos_salida);
	
	/*	Obtenemos el nombre del que hace de primario */

	const char* nombreDelPrimario=obtenerNombreParticipanteEnRelacion(
		elemento_relacion, "primario", "sistemaprimario");
	/*******************************************/
	/*Ahora obtenemos el nombre del secundario */
	/*******************************************/
	
	const char* nombreDelSecundario=obtenerNombreParticipanteEnRelacion(
		elemento_relacion, "secundario", "sistemasecundario");
	
	/*Aqui tenemos ya los nombres del primario y secundario */
	wxLogMessage ("El primario es %s y el secundario %s", 
		nombreDelPrimario, nombreDelSecundario);
		
	wxString nombrePrimario(nombreDelPrimario);
	wxString nombreSecundario(nombreDelSecundario);
	wxString entrada("Entrada");
	wxString salida("Salida");
	if ( (nombrePrimario!=entrada) && (nombreSecundario!=salida))
	{
		Elemento* elemento_primario=sistemaDevuelto->getRefElementoPorNombre
			(nombrePrimario.c_str());
		Elemento* elemento_secundario=sistemaDevuelto->getRefElementoPorNombre
			(nombreSecundario.c_str());
		assert (elemento_primario!=NULL);
		assert (elemento_secundario!=NULL);
		
		elemento_primario->anadirSuscriptor (elemento_secundario, pos_entrada, pos_salida);
	}
	return ;
}
Sistema* Ventana::ProcesarXML(DOMNode* elementoRaizXML, wxWindow* pagina)
{
	assert (elementoRaizXML!=NULL);
	
	Sistema* sistemaDevuelto =new Sistema();

	char* nombre = XMLString::transcode (elementoRaizXML->getNodeName());
	wxLogMessage ("Ventana::ProcesarXML");
	wxLogMessage ("Procesando elemento %s", nombre);

	DOMNode*	raiz= elementoRaizXML; 
	
	/*Vamos al primer hijo...*/
	DOMNode*	primer_elemento;
	primer_elemento= raiz->getFirstChild();
	/* 	Este primer hijo contiene el elemento <nombresistema>
		que recogemos y luego...*/
	primer_elemento = primer_elemento->getNextSibling();
	DOMNode* nombre_sistema = primer_elemento->getFirstChild(); /*Aqui esta el contenido*/
	
	const char* nombresistema=XMLString::transcode (nombre_sistema->getNodeValue());
	sistemaDevuelto->setNombreElemento (nombresistema);
	
	/*...recorremos todos los hijos*/
	DOMNode*	elemento_siguiente=primer_elemento->getNextSibling();
	
	/* 	Este bucle va recorriendo el xml. Si encuentra una relacion
		llama a una funcion especfica y si encuentra otra cosa
		llama a una fabrica de elementos que crea los elementos del
		tipo adecuado. El elemento creado se aade al sistema que
		se ver en esa pgina */
	while (elemento_siguiente!=NULL)
	{
		if (elemento_siguiente->getNodeType()==DOMNode::ELEMENT_NODE)
		{
			char* nombreetiquetaXML=XMLString::transcode
				(elemento_siguiente->getNodeName());
			wxLogMessage ("Procesando %s", nombre);
			DOMElement* elemento_A_Pasar=(DOMElement*) elemento_siguiente;
			
			if (strcmp(nombreetiquetaXML, "relacion")==0)
			{
				CrearRelacion (elemento_A_Pasar, sistemaDevuelto);
			}
			else
			{
				ElementoVisual* elementoVisual;
				elementoVisual = ObtenerElementoVisual(
					nombreetiquetaXML, pagina, elemento_A_Pasar, sistemaDevuelto);
				sistemaDevuelto->anadirElemento (elementoVisual);
			}

		} /* Fin del if que comprueba si es un nodo elemento */
		elemento_siguiente=elemento_siguiente->getNextSibling();
	} /* Fin del while que recorre los elementos */
	wxLogMessage ("El sistema se ha verificado");
	return sistemaDevuelto;
}
