#include "elemento.h"
#include "math.h"
#include "senal.h"
#include <iostream>

using namespace std;

Elemento::Elemento()
{
	Senal* senalVacia=new Senal();
	for (int i=0;i<MAX_LONG_VECTOR_ENTRADAS;i++)
	{
		entradas[i]=senalVacia;
	}
	
	for (int i=0;i<MAX_LONG_VECTOR_SALIDAS;i++)
	{
		salidas[i]=senalVacia;
	}
	for (int i=0; i<MAX_SUSCRIPCIONES; i++)
	{
		suscripciones[i]=NULL;
	}
	/* Esto debe cambiar en otros constructores de clases hija*/
	totalEntradas     	= POS_ENTRADA+1;
	totalSalidas  	  	= POS_SALIDA+1;
	totalSuscripciones 	= 0;
	elementoPadre		= NULL;
	strcpy (nombreElemento, "Elemento");
	
	string cad_entrada("Entrada estndar");
	string cad_salida("Salida estndar");
	nombreEntradas[cad_entrada]=POS_ENTRADA;
	nombreSalidas[cad_salida]=POS_SALIDA;

	/*	Los elementos que no tengan 1 entrada y 1 salida deben
		llamar en su constructor a este mtodo.*/
	inicializarCorrespondencias();
	
	pos_de_la_primera_entrada_en_sistema	=-1;
	pos_de_la_primera_salida_en_sistema		=-1;
}

Elemento::~Elemento()
{
//	printf ("Nombre:%s, destructor ~Elemento\n", nombreElemento);
	
	vector<Elemento*>::iterator i;
	/*	Se ordena a todos los elementos que avisan a este elemento, de
		que vamos a pasar a mejor vida y que deben eliminarnos como
		suscriptores */
	for (i=avisadores.begin(); i!=avisadores.end(); i++)
	{
		printf ("Avisando...\n");
		Elemento* avisador_concreto=*i;
		avisador_concreto->notificarEliminacion();
	}
}
void Elemento::setSistemaPadre (Elemento* padre)
{
	this->elementoPadre = padre;
}
Elemento* Elemento::getSistemaPadre()
{
	return this->elementoPadre;
}
int Elemento::setEntrada(Senal* entrada, int posicion)
{
	bool senal_es_nula = (entrada == NULL );
	assert (!senal_es_nula);
	assert (posicion>=0);
//	printf ("setEntrada:Ha llegado la senal %s al elemento %s, pos %i, direccion %x\n", entrada->toString(),
//		nombreElemento, posicion, this);
	if ( (posicion<0) || (posicion>MAX_LONG_VECTOR_ENTRADAS) )	
	{
		/*Error, se puso un valor incorrecto como posicion*/
		return POS_ENTRADA_INVALIDA; 
	}
	
	
	if (entrada->getTipo()!=entradas[posicion]->getTipo() )
	{
		/*Error, se introdujo p.ej un voltaje cuando se esperaba un magnetismo*/
		return TIPOS_NO_COINCIDEN;
	}
	assert (totalSuscripciones>=0);
	/*	Si todo est bien se apunta la nueva entrada y se notifica 
		a quien corresponda*/
	entradas[posicion]=entrada;
	//manejarEntrada(entrada, posicion);
	notificar();
	return ENTRADA_CAMBIADA;
}

void Elemento::manejarEntrada (Senal* senal, int posicion)
{
	printf ("-------------------------\n");
	printf ("Estamos en el elemento %s\n", nombreElemento);
	printf ("Alguien ha puesto una senal %s en la posicion %i\n",
		senal->toString(), posicion);
	
	printf ("-------------------------\n");
}



/*	Si alguien intenta introducir un nombre demasiado largo se truncara */
void Elemento::setNombreElemento (const char *nombre)
{
	if (nombre==NULL) return ;
	
	if (strlen(nombre)>MAX_LONG_NOMBRE_ELEMENTO)
	{
		strncpy (nombreElemento,nombre, MAX_LONG_NOMBRE_ELEMENTO);
	}
	else {
		strcpy (nombreElemento, nombre);
	}
}

const char* Elemento::getNombreElemento ()
{
	return nombreElemento;
}

int Elemento::getTotalEntradas ()
{
	return totalEntradas;
}

int Elemento::getTotalSalidas ()
{
	return totalSalidas;
}

int Elemento::getTotalSuscripciones ()
{
	return totalSuscripciones;
}


Suscripcion** Elemento::getSuscripciones()
{
	return suscripciones;
}
int Elemento::getTipoEntrada (int posicion)
{
	if ((posicion>=0) && (posicion<totalEntradas))
	{
		assert (entradas[posicion]!=NULL);
		return entradas[posicion]->getTipo();
	}
	else
	{
		return -1;
	}
}

int Elemento::getTipoSalida (int posicion)
{
	if ( (posicion>=0) && (posicion<totalSalidas) )
	{
		assert (salidas[posicion]!=NULL);
		return salidas[posicion]->getTipo();
	}
	else
	{
		return -1;
	}
}


Senal* Elemento::getEntradaPorPosicion(int posicion)
{
	
	if ( (posicion>=0) && (posicion<totalEntradas) )
	{
		return entradas[posicion];
	}
	else
	{
		return NULL;
	}
	
}
Senal* Elemento::getSalidaPorPosicion(int posicion)
{
	if ( (posicion>=0) && (posicion<totalSalidas) )
	{
		return salidas[posicion];
	}
	else
	{
		return NULL;
	}
	
}

int Elemento::getPrimeraEntradaUtil()
{
	int primeraEntradaUtil=POS_ENTRADA;
	return primeraEntradaUtil;
}

int Elemento::getPrimeraSalidaUtil()
{
	int primeraSalidaUtil=POS_SALIDA;
	return primeraSalidaUtil;
}

/*	Si los nombre pedidos coinciden con el nombre de este elemento
	devolvemos las entradas utiles*/
int Elemento::getPrimeraEntrada(const char* nombre)
{
	int cod_comparacion;
	cod_comparacion=strcmp(nombre, nombreElemento);
	if (cod_comparacion==0)
	{
		return POS_RESET;
	}
	else {
		return ELEMENTO_NO_ENCONTRADO;
	}
}

int Elemento::getPrimeraSalida (const char* nombre)
{
	int cod_comparacion;
	cod_comparacion=strcmp(nombre, nombreElemento);
	if (cod_comparacion==0)
	{
		return POS_SALIDA;
	}
	else {
		return ELEMENTO_NO_ENCONTRADO;
	}
}

/*	Si los nombre pedidos coinciden con el nombre de este elemento
	devolvemos las entradas utiles*/
int Elemento::getPrimeraEntradaUtil(const char* nombre)
{
	int cod_comparacion;
	cod_comparacion=strcmp(nombre, nombreElemento);
	if (cod_comparacion==0)
	{
		return POS_ENTRADA;
	}
	else {
		return ELEMENTO_NO_ENCONTRADO;
	}
}

int Elemento::getPrimeraSalidaUtil (const char* nombre)
{
	int cod_comparacion;
	cod_comparacion=strcmp(nombre, nombreElemento);
	if (cod_comparacion==0)
	{
		return POS_SALIDA;
	}
	else {
		return ELEMENTO_NO_ENCONTRADO;
	}
}



int Elemento::anadirSuscriptor (Elemento* e,int pos_entrada,int pos_salida)
{
	/* Se comprueban todas las circunstancias...*/
	if ( (pos_entrada<0) || (pos_entrada>MAX_LONG_VECTOR_ENTRADAS) )
	{
		return POS_ENTRADA_INVALIDA;
	}
	
	if ( (pos_salida<0) || (pos_salida>MAX_LONG_VECTOR_SALIDAS) )
	{
		return POS_SALIDA_INVALIDA;
	}
	
	if (e==NULL) return ELEMENTO_INVALIDO;
	
	/*	Esto no se puede comprobar aqu, ya que cuando se anade un elemento
		a un sistema, el elemento aadido se convierte en un suscriptor de
		ciertas entradas del sistema. Si se hiciera una comprobacin de tipos
		aqu habra fallos por puntero NULL, ya que en un principio hay conectores
		NULL en el sistema
		
	if (e->getTipoEntrada(pos_entrada)!=this->getTipoSalida(pos_salida))
	{
		return TIPOS_NO_COINCIDEN;
	}
		Fin del comentario */
//	printf ("Suscripcion: salida %i de %s hacia entrada %i de %s\n", pos_salida,
//		nombreElemento,pos_entrada, e->getNombreElemento());
	if (totalSuscripciones==MAX_SUSCRIPCIONES) return DEMASIADAS_SUSCRIPCIONES;
	
	/*	Si se dan todas las condiciones necesarias se toma nota de la nueva
		suscripcion en el vector de suscripciones */
	
	Suscripcion *s=new Suscripcion();
	s->setElementoSuscriptor(e);
	s->setElementoAvisador (this);
	s->setEntradaSuscriptor (pos_entrada);
	s->setSalidaAvisador (pos_salida);
	
	suscripciones[totalSuscripciones++]=s;
	/*	Tambien se informa al elemento de que nos tiene que
		anotar como avisador. l lo usar para poder notificarnos
		su destruccin. Sin esto, la eliminacin de elementos
		sera bastante complicada */
	e->anotarAvisador(this);
//#define DEBUG_SISTEMA
#ifdef DEBUG_SISTEMA
	printf ("Nueva suscripcion: Emisor %s(%i) y receptor %s(%i)\n",
			this->getNombreElemento(),pos_salida,
			e->getNombreElemento(),pos_entrada);
#endif 
	return 0;
}

int Elemento::eliminarSuscriptor (Elemento* e,int pos_entrada,int pos_salida)
{
	Elemento* elementoActual;
	int salidaDelElemento;
	int entradaDelElemento;
	Suscripcion* suscripcionActual;
	
	/*	Para cada suscripcin*/
	for (	int posSuscripcion=0;
			posSuscripcion < totalSuscripciones;
			posSuscripcion++ )
	{
		suscripcionActual = suscripciones[posSuscripcion];
		
		/*	Obtenemos el elemento suscriptor*/
		elementoActual = suscripcionActual->getElementoSuscriptor();
		
		/*	Obtenemos la posicion por donde le entra el aviso*/
		entradaDelElemento = suscripcionActual->getPosEntradaSuscriptor();
		
		/*	Obtenemos la salida del elemento */
		salidaDelElemento = suscripcionActual->getPosSalidaAvisador();
		
		/*	Comprobamos si el suscriptor es el que nos han pasado*/
		bool suscriptorCorrecto= (elementoActual==e);
		
		/*	Comprobamos si la posicion de entrada es la misma*/
		bool entradaCorrecta= (entradaDelElemento==pos_entrada);
		
		/*	Comprobar si la salida del avisador es la misma*/
		bool salidaCorrecta= (salidaDelElemento==pos_salida);
		
		/*	Se comprueba si todo coincide*/
		bool todoCoincide=(	suscriptorCorrecto && entradaCorrecta &&
							salidaCorrecta);
		
		/*Si todo coincide...*/
		if (todoCoincide)
		{
			/*Se borra esta suscripcion*/
			e->eliminarAvisador(this);
			borrarSuscripcion(posSuscripcion);
			return SUSCRIPCION_BORRADA;
		}
	} /*Fin del for*/
	
	/*	No se encontro la suscripcion*/
	return SUSCRIPCION_NO_EXISTIA;
}

int Elemento::eliminarSuscripcionesAElemento (Elemento *e)
{
	bool elemento_es_nulo = (e==NULL);
	assert (!elemento_es_nulo);
	
	int codigo = NO_SE_ENCONTRO_NADA_PARA_ELIMINAR;
	
	/*	Se buscan suscripciones que tengan como suscriptor (receptor)
		al elemento que nos pasan. Si se encuentran, se elimina
		esa suscripcion */
	for (int i=0; i< MAX_SUSCRIPCIONES; i++)
	{
		if (suscripciones[i]!=NULL)
		{
			if (suscripciones[i]->getElementoSuscriptor()== e)
			{
				codigo = SUSCRIPCION_ELIMINADA;
				/* Descomentar esto para ver la progresion de
					borrado de suscripciones */
				#define DEBUG_ELIMINAR_SUSCRIPCIONES
				#ifdef DEBUG_ELIMINAR_SUSCRIPCIONES
					printf ("-->Borrando suscripcion ...\n");
					printf ("El emisor se llama %s\n", nombreElemento);
					printf ("El suscriptor se llama %s\n",
						e->getNombreElemento());
					printf ("La salida emisora era la %i\n",
						suscripciones[i]->getPosSalidaAvisador());
					printf ("La entrada receptora era la %i\n", 
						suscripciones[i]->getPosEntradaSuscriptor());
				#endif
				delete suscripciones[i];
				suscripciones[i]=NULL;
				#ifdef DEBUG_ELIMINAR_SUSCRIPCIONES
					printf ("Suscripcion borrada\n");
				#endif
			}
		}
	}
	return codigo;
}
void Elemento::notificar()
{
	Elemento* elem_a_notificar;
	int pos_entrada_a_notificar;
	int pos_salida_a_notificar;
	assert (totalSuscripciones>=0);
	for (int i=0;i<totalSuscripciones;i++)
	{
		/*	Puede haber muchas suscripciones y puede haber huecos
			que hayan sido rellenados con NULL*/
		if (suscripciones[i]==NULL) continue;
		assert (suscripciones[i]!=NULL);
		elem_a_notificar		=suscripciones[i]->getElementoSuscriptor();
		pos_entrada_a_notificar	=suscripciones[i]->getPosEntradaSuscriptor();
		pos_salida_a_notificar	=suscripciones[i]->getPosSalidaAvisador();
	
		elem_a_notificar->setEntrada
			(salidas[pos_salida_a_notificar], pos_entrada_a_notificar);
		//assert (codigo==ENTRADA_CAMBIADA);
//		#define DEBUG_BIESTABLE
#ifdef DEBUG_BIESTABLE
		printf ("%s propaga la (sal.%i) hacia %s(ent.%i, direccion %x) con valor %s!\n",
			this->getNombreElemento(), pos_salida_a_notificar,
			elem_a_notificar->getNombreElemento(),pos_entrada_a_notificar,
			elem_a_notificar,
			salidas[pos_salida_a_notificar]->toString());
#endif 
	}
}

void Elemento::pintarSuscripciones()
{
	printf ("%s tiene %i suscripciones\n", nombreElemento, totalSuscripciones);
	for (int i=0;i<totalSuscripciones;i++)
	{
		Elemento* avisador=this->suscripciones[i]->getElementoAvisador();
		Elemento* suscriptor=this->suscripciones[i]->getElementoSuscriptor();
		const char *nombreAvisador=avisador->getNombreElemento();
		const char *nombreSuscriptor=suscriptor->getNombreElemento();
		int entradaSuscriptor=this->suscripciones[i]->getPosEntradaSuscriptor();
		int salidaAvisador =this->suscripciones[i]->getPosSalidaAvisador();
		printf ("\t--%s saca por la salida %i una seal a la entrada %i de %s\n",
			nombreAvisador, salidaAvisador,  entradaSuscriptor, nombreSuscriptor);
	}
	
}

string Elemento::getInformacionElemento()
{
	string nombre(nombreElemento);
	string informacion;
	
	informacion=""+nombre+":\n";
	
	for (int i=0; i<totalSuscripciones; i++)
	{
		Elemento* suscriptor = suscripciones[i]->getElementoSuscriptor();
		
		informacion+= string("\tSuscriptor:")+
			string (suscriptor->getNombreElemento())+"\n";
	}
	return informacion;
}
bool Elemento::iguales(float x, float y)
{
	if (fabs (x-y)<MARGEN_ACEPTABLE) return true;
	return false;	
}

bool Elemento::iguales(Senal* s1, Senal* s2)
{
	assert (s1!=NULL);
	assert (s2!=NULL);
	const char* senal1=s1->toString();
	const char* senal2=s2->toString();
	int codigo = strcmp (senal1, senal2);
	if (codigo==0) return true;
	
	return false;
}
void Elemento::borrarSuscripcion (int posicion)
{
	/*	Este borrado no interfiere con el liberado de memoria de WxWidgets*/
	delete suscripciones[posicion]; 
	
	/*	Se desplazan todas las suscripciones hacia la izquierda*/
	for (int posVectorSuscripciones=posicion;
			posVectorSuscripciones<totalSuscripciones;
			posVectorSuscripciones++)
	{
		suscripciones[posicion]=suscripciones[posicion+1];
	}
	
	totalSuscripciones--;
			
}



int Elemento::getX()
{
	return x;
}
int Elemento::getY()
{
	return y;
}

void Elemento::setX(int x)
{
	this->x=x;
}
void Elemento::setY(int y)
{
	this->y=y;
}

void Elemento::getXMLSuscriptores (DOMDocument *documento_xml)
{
	if (totalSuscripciones==0)
		return ;
		
	DOMElement* elemento_raiz=documento_xml->getDocumentElement();
	DOMElement* elemento;
	
	for (int i=0; i<totalSuscripciones; i++)
	{
		Suscripcion* suscripcion = suscripciones[i];
		if (suscripcion==NULL) {
			continue;
		}
		Elemento* avisador 		= suscripcion->getElementoAvisador();
		Elemento* suscriptor	= suscripcion->getElementoSuscriptor();
		int entrada_suscriptor 	= suscripcion->getPosEntradaSuscriptor();
		int salida_avisador		= suscripcion->getPosSalidaAvisador();

		elemento=documento_xml->createElement ( C ("relacion") );
		elemento_raiz->appendChild(elemento);
		
		elemento->setAttribute( C("pos_salida"),  C(salida_avisador) );
		elemento->setAttribute( C("pos_entrada"), C(entrada_suscriptor) );		
		
		DOMElement* elementoprimario	= documento_xml->createElement( C ("primario") );
		DOMElement* elementosecundario	= documento_xml->createElement( C ("secundario") );
		DOMText* textoNombrePrimario 	= documento_xml->createTextNode ( C (avisador->getNombreElemento()) );
		DOMText* textoNombreSecundario 	= documento_xml->createTextNode ( C (suscriptor->getNombreElemento()) );
		
		elementoprimario->appendChild(textoNombrePrimario);
		elementosecundario->appendChild(textoNombreSecundario);	
		
		elemento->appendChild(elementoprimario);
		elemento->appendChild(elementosecundario);	
		
		//
		
	}
}

map<string,int> Elemento::getNombreEntradas()
{
	/*	Se devuelve una versin actualizada para que
		el usuario pueda ver a quien pertenece la entrada */
	return actualizarMapaNombres(nombreEntradas);
}

map<string,int> Elemento::getNombreSalidas()
{
	return actualizarMapaNombres(nombreSalidas);
}

void Elemento::obtenerDatosComunes(DOMElement* nodo)
{
	x=getAtributo ( nodo, "x" );
	y=getAtributo ( nodo, "y" );
	/*	Una vez recuperada la posicion, recuperamos el nombre del
		nodo XML que se nos ha pasado */
	
	strncpy (nombreElemento, recuperarNombreElemento ( nodo ), 
		MAX_LONG_NOMBRE_ELEMENTO);
}


map<string, int> Elemento::actualizarMapaNombres (map<string, int> &mapa)
{
	/*	Aqui se meten los nuevos datos que se devolvern*/
	map<string, int> mapaADevolver;
	map<string, int>::iterator i;
	
	string cadenaDe( " de " );
	string nombreDeElemento(nombreElemento);
			
	for (i=mapa.begin(); i!= mapa.end(); i++)
	{
		/*	Se recuperan los datos */
		string 	nombreDeEntrada 	= i->first;
		int 	posicionDeEntrada	= i->second;
		
		/*	Se corrigen...*/
		nombreDeEntrada = nombreDeEntrada + cadenaDe + nombreElemento;
		/*	Y se realmacenan */
		mapaADevolver[nombreDeEntrada] = posicionDeEntrada;
	}	
	return mapaADevolver;
}

void Elemento::anotarAvisador		(Elemento* e)
{
	avisadores.push_back (e);
} /* Fin de anotar avisador */

void Elemento::eliminarAvisador 	(Elemento* e)
{
	vector<Elemento*>::iterator pos_elemento_buscado;
	
	pos_elemento_buscado = find (
			avisadores.begin(), avisadores.end(), e);
	/*	Si se ha encontrado el elemento, se elimina */
	if (pos_elemento_buscado!=avisadores.end())
	{
		avisadores.erase(pos_elemento_buscado);
	}
} /* Fin de eliminar avisador */

void Elemento::notificarEliminacion()
{
	vector<Elemento*>::iterator elemento_examinado;
	
	for (	elemento_examinado = avisadores.begin(); 
			elemento_examinado!= avisadores.end();
			elemento_examinado++)
	{
		Elemento* e=(*elemento_examinado);
		e->eliminarSuscripcionesAElemento(this);	
	}
}

Elemento* Elemento::getRefElementoPorNombre (const char* nombre)
{
	/*	Si el nombre de las cadenas coincide se devuelve una
		referencia, si no, se devuelve NULL*/
	if (0==strcmp(nombre, nombreElemento))
	{
		return this;
	}
	else return NULL;
}

Senal* Elemento::getEntradaPorNombreYPosicion (const char* nombre, int pos)
{
	if ( 0 == strcmp (nombre, nombreElemento))
	{
		return entradas[pos];
	}
	else return NULL;
}
		

Senal* Elemento::getSalidaPorNombreYPosicion (const char* nombre, int pos)
{
	if ( 0 == strcmp (nombre, nombreElemento))
	{
		return salidas[pos];
	}
	else return NULL;
}

int Elemento::getEntradaDeSistema (const char* nombrePasado, 	int posicion)
{
	assert (posicion>=0);
	assert (posicion<=totalEntradas);
	printf ("%s->getEntradaDeSistema(%s, %i)\n", 
		nombreElemento, nombrePasado, posicion);
	if ( 0 == strcmp (nombrePasado, nombreElemento))
	{
		/*	Con esto comprobamos que no busquen una posicion
			mas alla del los limites */
		if ( (0 < posicion) && (posicion<totalEntradas) )
		{
			return posicion;
		}
	}
	
	return -1;
}	
int Elemento::getSalidaDeSistema  (const char* nombrePasado, 	int posicion)
{
	assert (posicion>=0);
	assert (posicion<=totalSalidas);
	printf ("%s->getSalidaDeSistema(%s, %i)\n", 
		nombreElemento, nombrePasado, posicion);
	if ( 0 == strcmp (nombrePasado, nombreElemento))
	{
		/*	Con esto comprobamos que no busquen una posicion
			mas alla del los limites */
		if ( (0 < posicion) && (posicion<totalSalidas) )
		{
			return posicion;
		}
	}
	return -1;
}

bool Elemento::entradasEstanMapeadas()
{
	if (pos_de_la_primera_entrada_en_sistema==-1)
		return false;
	
	return true;
}

bool Elemento::salidasEstanMapeadas()
{
	if (pos_de_la_primera_salida_en_sistema==-1)
		return false;
	
	return true;
}

void Elemento::setMapeoPrimeraEntrada(int pos_sistema)
{
	this->pos_de_la_primera_entrada_en_sistema=pos_sistema;
}
void Elemento::setMapeoPrimeraSalida(int pos_sistema)
{
	this->pos_de_la_primera_salida_en_sistema=pos_sistema;
}
bool Elemento::esVisible()
{
	return true;
}
void Elemento::inicializarCorrespondencias()
{
	for (int i=0; i<totalEntradas; i++)
	{
		/*	En los elementos simples, no en los sistemas,
			las correspondencias de cada posicin se da
			consigo misma, de ahi que se use i dos veces */
		Correspondencia c(this, i, i, NULL);
		correspondencia_de_entradas.push_back(c);
//		printf ("Correspondencia de entrada %i\n", i);
	}
	
	for (int i=0; i<totalSalidas; i++)
	{
		/*	En los elementos simples, no en los sistemas,
			las correspondencias de cada posicin se da
			consigo misma, de ahi que se use i dos veces */
		Correspondencia c(this, i, i, NULL);
		correspondencia_de_salidas.push_back(c);
//		printf ("Correspondencia de salida %i\n", i);
	}
}

