#include "lampara.h"
#include "constantes.h"
#include <iostream>
using namespace std;

Lampara::Lampara ()
{
	this->v1=NULL;
	this->v2=NULL;
	this->l1=NULL;
	this->l2=NULL;	
	entradas[POS_ENTRADA] = NULL;
	salidas [POS_SALIDA]  = NULL;

}
Lampara::Lampara(Voltaje* v1, Luminosidad* l1, Voltaje* v2, Luminosidad* l2)
{
	this->v1=v1;
	this->v2=v2;
	this->l1=l1;
	this->l2=l2;	
	entradas[POS_ENTRADA] = v1;
	salidas [POS_SALIDA]  = l1;
}

Voltaje* Lampara::getVoltajeReposo()
{
	return v1;
}

Voltaje* Lampara::getVoltajeActivacion()
{
	return v2;
}

Luminosidad* Lampara::getLuminosidadReposo()
{
	return l1;
}

Luminosidad* Lampara::getLuminosidadActivacion()
{
	return l2;
}

int	Lampara::setEntrada (Senal* s, int posicion)
{
	Voltaje* v=(Voltaje*) s;
#ifdef DEBUG_LAMPARA
	printf ("El voltaje recibido es %f\n", v->getValor() );
#endif
	if (v2->getValor()==v->getValor() )
	{
#ifdef DEBUG_LAMPARA
		printf ("Activo la lampara\n");
#endif

		salidas[POS_SALIDA]=l2;
		return (Elemento::setEntrada( s, posicion ));
	}
	if (v1->getValor()==v->getValor() )
	{
#ifdef DEBUG_LAMPARA
		printf ("Desactivo la lampara\n");
#endif
		salidas[POS_SALIDA]=l1;
		return (Elemento::setEntrada( s, posicion ));
	}
	/*	Si se llega a este punto es porque se
		pas una entrada que no produce efecto */
	return 0;
}

NodoXML Lampara::getXML ( DOMImplementation *impl )
{
	//etqElemento se hereda por parte de XMLSerializable	
	etqElemento = XMLString::transcode ( "lampara" );
	DOMDocument* doc = impl->createDocument();
	cout << "Creando documento base XML para lampara\n";
	doc = getDocumentoXMLBase ( doc, "lampara", nombreElemento, x, y );
	
	/*	Ahora aadimos a doc los elementos XML para indicar las distintas
		caractersticas. No se debe alterar el orden de creacin de elementos
		ya que el esquema exige dicho orden */
	cout << "Documento base lampara creado\n";
	anadirElementoXML (doc, "entradareposo", 		
								getVoltajeReposo() 			);
	anadirElementoXML (doc, "salidareposo", 		
								getLuminosidadReposo()			);
	cout << "XML lampara:Reposo creados\n";
	anadirElementoXML (doc, "entradaactivacion",	
								getVoltajeActivacion() 		);
	anadirElementoXML (doc, "salidaactivacion",		
								getLuminosidadActivacion() 	);
	cout << "XML lampara:Activacion creada\n";
	
	
	DOMDocumentFragment* fragmento = doc->createDocumentFragment ();
	fragmento->appendChild ( doc->getDocumentElement() );
	return fragmento;
}



void Lampara::construir (DOMElement* nodo)
{
	obtenerDatosComunes (nodo);
	/* Obtenemos los valores de los distintos nodos */
	v1 = (Voltaje*) 		getSenal ( VOLTAJE, "entradareposo", nodo);
	v2 = (Voltaje*) 		getSenal ( VOLTAJE, "entradaactivacion", nodo);
	l1 = (Luminosidad*)		getSenal ( LUMINOSIDAD, "salidareposo", nodo);
	l2 = (Luminosidad*) 	getSenal ( LUMINOSIDAD, "salidaactivacion", nodo);
	
	entradas[POS_ENTRADA]	=v1;
	salidas[POS_SALIDA]		=l1;
	
}
