#ifndef _parser_
#define _parser_
/*Fichero parser.h

	Contiene la declaracin de la clase parser, encargada de manipular
	ficheros XML, validndolos y extrayendo de l objetos que componen
	un sistema a simular
*/

#include <iostream>
#include <vector>
#include "elemento.h"
#include "sistema.h"
#include "nodoxml.h"
//Evitar instanciacin de plantillas
#define XERCES_TMPLSINC

#include <xercesc/parsers/XercesDOMParser.hpp>
#include <xercesc/dom/DOM.hpp>
#include <xercesc/sax/HandlerBase.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/framework/LocalFileInputSource.hpp>

#ifdef XERCES_CPP_NAMESPACE_USE
XERCES_CPP_NAMESPACE_USE
#endif 

using namespace std;


class Parser {
	public:
		Parser ();
		~Parser();
		/*Este metodo nos comprueba si un determinado fichero en XML
		 *es vlido o no*/
		bool 		esValido (const char* fichero);
		
		/*	Nos dice si ha habido errores en el ltimo anlisis*/
		bool 		haHabidoErrores();
		
		char*		obtenerError();
		/*Este metodo extrae del fichero XML los elementos
		 *del sistema que queremos simular*/
		vector <Elemento*> 		extraerElementos 	();
		vector <Suscripcion*> 	extraerRelaciones	();
		
		/*	Esta funcion nos devuelve el elemento Sistema que
			pueda haber en un documento */
		Sistema* getSistema ();
				
		/*	Suponiendo que ya se haya suministrado un documento este
			mtodo se sita al comienzo del documento */
		DOMNode* 		inicializarRecorrido();
		
		/*Esta funcion escribe un elemento determinado en un fichero*/
		void				escribir (Elemento* elemento, char *fichero);
	private:
		DOMBuilder* 			parser;
		DOMDocument* 			documento;
		DOMErrorHandler* 		manejadorErrores;
		
		/* 	Este nodo apunta al nodo raiz del documento XML*/
		DOMNode*				nodoRaiz;
		
		/*	Este es el nodo XML que se est leyendo*/
		DOMNode*				elementoActual;
};
#endif
