#include "unit++.h"
#include "pulsador.h"
#include "zumbador.h"
#include "elemento.h"
#include "temporizador.h"
#include "constantes.h"


using namespace unitpp;

namespace
{
	class PruebaTemporizador: public suite
	{
		void PruebaSimpleTemporizador()
		{
			Voltaje *vSalida;
			
			Voltaje* vReposo;
			Voltaje* vActivacion;
			Voltaje* vSalidaReposo;
			Voltaje* vSalidaActivacion;
			
			
		
			vReposo					=	new Voltaje(0.0);
			vSalidaReposo			=	new Voltaje(0.0);	
			vActivacion				=	new Voltaje(5.0);
			vSalidaActivacion		=	new Voltaje(5.0);
			
			/*	Los temporizadores esperan 5 intervalos de tiempo por defecto */	
			unsigned long tiempo	= 	5;
						
			Temporizador* elemento = new Temporizador (
				vReposo, vSalidaReposo, vActivacion, vSalidaActivacion, tiempo
			);
			
			/*	Comprobamos la salida antes */
			
			vSalida = (Voltaje*) elemento->getSalidaPorPosicion();
			/*	Que en principio debe tener el mismo valor 
				que la salida en reposo que le hemos puesto */
			assert_eq ("Salida temporizador en reposo", 
				 vSalidaReposo->getValor(), vSalida->getValor());
				
				
			/*	Se pone la entrada en el temporizador */
			elemento->setEntrada (vActivacion);
			
			/*	Y se actualiza unas pocas veces */
			for (int i=0; i<5; i++)
			{
				elemento->actualizar();
			}
			
			/*	Ahora la salida deberia cambiar */	
			vSalida = (Voltaje*) elemento->getSalidaPorPosicion();
			/*	Y en principio debera tener el mismo valor 
				que la salida en reposo que le hemos puesto */
			assert_eq ("Salida temporizador en activacion", 
				vSalidaActivacion->getValor(), vSalida->getValor() );
		}
		void PruebaPulsadorTemporizador ()
		{
			Conexion* c1=new Conexion (CONEXION_INACTIVA);
			Conexion* c2=new Conexion (CONEXION_ACTIVA);
			Voltaje*  v1=new Voltaje  (0.0);
			Voltaje*  v2=new Voltaje  (5.0);
			
			Pulsador* p=new Pulsador(c1,v1,c2,v2);
			
			Voltaje* vReposo;
			Voltaje* vActivacion;
			Voltaje* vSalidaReposo;
			Voltaje* vSalidaActivacion;
			
			
		
			vReposo					=	new Voltaje(0.0);
			vSalidaReposo			=	new Voltaje(0.0);	
			vActivacion				=	new Voltaje(5.0);
			vSalidaActivacion		=	new Voltaje(5.0);
			
			/*	Los temporizadores esperan 5 intervalos de tiempo por defecto */	
			unsigned long tiempo	= 	5;
						
			Temporizador* reloj = new Temporizador (
				vReposo, vSalidaReposo, vActivacion, vSalidaActivacion, tiempo
			);
			/*	Nombre un poco comico. Es que a veces me cansaba ;) */
			Frecuencia	*fReposo	=	new Frecuencia (0);
			Frecuencia	*fActivacion=	new	Frecuencia (3000);
			Zumbador *zumbazumba=new Zumbador (vReposo, fReposo, 
				vActivacion, fActivacion);

			p->anadirSuscriptor (reloj);
			reloj->anadirSuscriptor (zumbazumba);
			/*	Si se pulsa el boton...*/
			p->setEntrada (c2);
			
			/*	El temporizador deberia activarse despues de 5 pulsos */
			
			for (int i=0; i<5; i++)
			{
				reloj->actualizar();
			}
			
			Voltaje *vSalida = (Voltaje*) reloj->getSalidaPorPosicion();
			/*	Y en principio debera tener el mismo valor 
				que la salida en reposo que le hemos puesto */
			assert_eq ("Salida temporizador en activacion", 
				vSalidaActivacion->getValor(), vSalida->getValor() );
			
			/*	Y el zumbador deberia zumbar */
			
			Frecuencia* fSalida =(Frecuencia*) zumbazumba->getSalidaPorPosicion();
			assert_eq ("Salida zumbador tras el timeout", 
				fActivacion->getValor(), fSalida->getValor() );
		}
		public:
		PruebaTemporizador(): suite ("Prueba de temporizador")
		{
			add ("Prueba Temporizador", testcase 
			(this, "PruebaTemporizador", 
			&PruebaTemporizador::PruebaSimpleTemporizador));
			
			add ("Prueba Temporizador con pulsador", testcase 
			(this, "PruebaTemporizadorPulsador", 
			&PruebaTemporizador::PruebaPulsadorTemporizador));
			
			suite::main().add("PruebaTemporizador", this);
		}
	}; /*Fin de la clase */
	PruebaTemporizador *ptemp=new PruebaTemporizador();	
}
