#include "unit++.h"
#include "fotosensor.h"
#include "parser.h"

using namespace unitpp;

namespace
{
	
	class PruebaXMLFotoSensor : public suite
	{
		void PruebaValoresFotoSensor()
		{
			bool ficheroCorrecto = false;
			Parser* p=new Parser();
			Sistema* sistemaParseado;
			/* 	Se comprueba primero un fichero con un 
				sistema simple que contiene un electroiman*/
			ficheroCorrecto	=	p->esValido ("xml\\fotosensor.xml");
			if (ficheroCorrecto)
			{
				sistemaParseado=p->getSistema();
				cout << "Sistema XML FotoSensor obtenido" << endl;
				/* 	En este punto deberia haber un fotosensor dentro del
					sistema. El fotosensor de este fichero se llama 
					"FS1" (no modificar los ficheros de test del
					directorio xml) */
				FotoSensor* fs = (FotoSensor*)
					sistemaParseado->getRefElementoPorNombre("FS1");

				/* Construimos los valores a mano...*/	
				Voltaje*  	v1  = 	new Voltaje  (0.0);
				Voltaje*  	v2  = 	new Voltaje  (9.0);
				Luminosidad* l1	=	new Luminosidad("e3004c");
				Luminosidad* l2	=	new Luminosidad("000000");
				
				//Valores que leemos y compararemos con los construidos
				//para comprobar si son iguales
				Voltaje*	 vSalidaReposo;
				Voltaje*	 vSalidaActivacion;
				Luminosidad* lumEntradaReposo;
				Luminosidad* lumEntradaActivacion;
				
				
				/* 	...y comparo esos valores con los leidos del fichero...*/
				
				vSalidaReposo = fs->getVoltajeSalidaReposo();
				assert_eq ("(XML) Voltaje salida reposo en electroiman", 
					vSalidaReposo->getValor(), v1->getValor() );
					
				vSalidaActivacion = fs->getVoltajeSalidaActivacion();
				assert_eq ("(XML) Voltaje salida activacion en electroiman", 
					vSalidaActivacion->getValor(), v2->getValor() );
				
				lumEntradaReposo 	 = fs->getLuminosidadEntradaReposo() ;
				lumEntradaActivacion = fs->getLuminosidadEntradaActivacion() ;
				
				/* 	Se comprueba si las seales construida y la leida
					del XML son iguales...*/
				bool sonIguales;
				
				sonIguales=lumEntradaReposo->esIgualA ( l1 );
				assert_eq ("(XML) Luminosidad en reposo", 
					true, sonIguales );
					
				sonIguales=lumEntradaActivacion->esIgualA ( l2 );
				assert_eq ("(XML) Luminosidad en activacion", 
					true, sonIguales );
			
				/*	Comprobamos la posicion del fotosensor.xml
					para comprobar si es la correcta */
				unsigned int x, y;
				
				x=fs->getX();
				y=fs->getY();
				
				assert_eq ("(XML) Posicion X del FotoSensor", 40, x);
				assert_eq ("(XML) Posicion Y del FotoSensor", 275, y);
				p->escribir (fs, "xml-resultadoTest-xmlfotosensor.xml");
				delete p;
					
			}	/* Fin del if que comprueba si el fichero es correcto */
		}
		public:
			PruebaXMLFotoSensor() : suite ("Prueba del XML del fotosensor")
			{
				add ("PruebaXMLFotoSensor", testcase (this,
					"PruebaXMLFotoSensor", 
					&PruebaXMLFotoSensor::PruebaValoresFotoSensor) );
				
				suite::main().add("PruebaXMLFotoSensor", this);
			}
	};
	PruebaXMLFotoSensor *prFotoSensor=new PruebaXMLFotoSensor();	
}
